/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#include "PluginLib.h"
#include "PluginLibCommon.h"
#include "Counters.h"

void GlobalLog(const char* fmt, ...);

extern NotificationCounters g_nc;
BOOL g_fActive = 0;

static void PluginProc(PHTTP_FILTER_CONTEXT pfc, DWORD notificationType, LPVOID pvNotification)
{
#if 0
    if (!g_fActive)
    {
        if (SF_NOTIFY_END_OF_REQUEST == notificationType)
        {
            g_fActive = TRUE;
        }
        return;
    }
#endif
    switch (notificationType)
    {
    case SF_NOTIFY_SEND_RESPONSE:
        g_nc.Inc(2);
        DebugString("SEND_RESPONSE");
        OnSendResponse(pfc, (PHTTP_FILTER_SEND_RESPONSE)pvNotification);
        break;
    
    case SF_NOTIFY_SEND_RAW_DATA:
        g_nc.Inc(3);
        DebugString("RAW_DATA");
        OnRawData(pfc, (PHTTP_FILTER_RAW_DATA)pvNotification);
        break;

    case SF_NOTIFY_END_OF_REQUEST:
        g_nc.Inc(4);
        DebugString("END_REQUEST");
        OnEndRequest(pfc);
        break;
    }
}

static volatile BOOL g_fCrashed = FALSE;
static volatile BOOL g_fInitSuccess = FALSE;
static volatile LONG g_fInitialized = FALSE;

DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc, DWORD notificationType, LPVOID pvNotification)
{
    g_nc.Tick();
    g_nc.Inc(0);
    if (g_fInitSuccess && !g_fCrashed)
    {
        g_nc.Inc(1);
        __try
        {
            PluginProc(pfc, notificationType, pvNotification);
        }
        __except (EXCEPTION_EXECUTE_HANDLER)
        {
            g_fCrashed = TRUE;
            GlobalLog("Plugin crashed");
        }
    }
    return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
{
    DebugString("GetFilterVersion()");

    if (0 == InterlockedCompareExchange(&g_fInitialized, 1, 0))
    {
        ConfigPluginGlobals config;
        config.SetDefault();
        g_fInitSuccess = InitializePlugin(config, NULL); // can't do this in DllMain because GetUserName() is in advapi32.lib
    }
    
    pVer->dwFilterVersion = HTTP_FILTER_REVISION;
    strncpy_s(pVer->lpszFilterDesc, sizeof(pVer->lpszFilterDesc), "SSP IIS Plugin", _TRUNCATE);
    pVer->dwFlags = 0;
    pVer->dwFlags |= SF_NOTIFY_SEND_RESPONSE;
    pVer->dwFlags |= SF_NOTIFY_SEND_RAW_DATA;
    pVer->dwFlags |= SF_NOTIFY_END_OF_REQUEST;
    pVer->dwFlags |= SF_NOTIFY_NONSECURE_PORT;
    return TRUE;
}

BOOL WINAPI TerminateFilter(DWORD dwFlags)
{
    UNREFERENCED_PARAMETER(dwFlags);
    if (g_fInitialized && g_fInitSuccess && !g_fCrashed)
    {
        FreePlugin(NULL);
    }
    return TRUE;
}

