/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "basic_types.h"
#include "shared.h"
#include "zlib.h"
#include "diskio.h"

/*************************************************************************
**  Class: CompressingFileWriter
**
**  Description: Compressing sequential file writer class.  
**      Efficiently writes the file
**      sequentially using unbuffered and asynch I/O.  
**
**************************************************************************/
class CompressingFileWriter
{
private:
    SEQWRITE m_seqwrite;

    void* m_pOwnMemory;

    // buffer for compressed data
    UInt8 *m_compressedBuffer;

    z_stream m_strmCompress;

    bool m_fCompression;
    bool m_fInitialized;
    bool m_fOpened;

    static void *zlib_alloc(void *opaque, UINT items, UINT size);
    static void zlib_free(void *opaque, void *address);
    void ResetCompressedOutput();
    void ResetCompressedInput();
    void ResetCompression();
    DWORD32 InitializeCompression(IAllocator* pAllocator,
                              BOOL fGZip);
    static size_t DeflateMemorySize();
    static size_t MemoryNeeded(BOOL fCompression);
public:
    CompressingFileWriter();
    ~CompressingFileWriter();
    void Construct();
    DWORD32 DoDispose( void );

    DWORD32 DoInit(BOOL fCompression = TRUE,
                   BOOL fGZip = FALSE,
                   IAllocator* pAllocator = NULL,
                   int cbWrite = SEQWRITE::c_writeBufSizeDefault, 
                   int maxWrites = SEQWRITE::c_maxWritesDefault);

    DWORD32 Open(const TCHAR* wszFile);
    DWORD32 Close();
    
    DWORD32 Write(const void* pbWrite, DWORD cbWrite);
    DWORD32 Print(const char* szFormat, ...);

    SEQWRITE *GetWriter(void) { return &m_seqwrite; };

    static DWORD32 CompressFile(const TCHAR *pszUncompressed, const TCHAR *pszCompressed);
    
    DWORD32 InitAndOpen                 (const TCHAR* wszFile, BOOL fCompression = TRUE, BOOL fGZip = FALSE);
    static CompressingFileWriter* Create(const TCHAR* wszFile, BOOL fCompression = TRUE, BOOL fGZip = FALSE);

    const static UInt32 c_compressedBufferSize = 64 * 1024;
};
