/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#include "DeltaFile.h"

bool GenerateDeltaFileEntry(unsigned flags, Signature sig, VisitCount visitCount, const char* pbUrl, size_t cbUrl, __out_bcount_part(*pcbBuf,*pcbBuf) char* buf, size_t* pcbBuf)
{
    UInt16 cb = (UInt16)cbUrl;
    if (cb != cbUrl)
    {
        return false;
    }
    if (cb > *pcbBuf - sizeof(flags) - sizeof(sig) - sizeof(visitCount) - sizeof(cb))
    {
        return false;
    }

    char* p = buf;
    *((unsigned*)p) = flags;  p += sizeof(flags);
    *((Signature*)p) = sig;  p += sizeof(sig);
    *((VisitCount*)p) = visitCount;  p += sizeof(VisitCount);
    *((UInt16*)p) = cb;  p += sizeof(UInt16);
    if (cb) // 0-length URL may be used as EOF marker
    {
        memcpy(p, pbUrl, cb); p += cb;
    }
    *pcbBuf = p - buf;
    return true;
}

bool ReadDeltaFileEntry(IFileReader* pFile, unsigned* pFlags, Signature* pSig, VisitCount* pVisitCount, __out_bcount_part(*pcbUrl,*pcbUrl) char* pbUrl, size_t* pcbUrl)
{
    *pFlags = 0;
    if (pFile->Version() > 0)
    {
        if (!pFile->Read(pFlags, sizeof(*pFlags))) return false;
    }
    if (!pFile->Read(pSig, sizeof(Signature))) return false;
    if (!pFile->Read(pVisitCount, sizeof(VisitCount))) return false;
    UInt16 cb;
    if (!pFile->Read(&cb, sizeof(cb))) return false;
    if (cb > *pcbUrl) return false;
    if (cb) // 0-length URL may be used as EOF marker
    {
        if (!pFile->Read(pbUrl, cb)) return false;
    }
    *pcbUrl = cb;
    return true;
}

int DeltaFileCppLineCount()
{
    return __LINE__;
}
