/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "basic_types.h"

class DiskSpaceTracker
{
public:
    DiskSpaceTracker();

    ~DiskSpaceTracker();

    BOOL DoInit(const TCHAR* wszFolder, unsigned uCheckPeriodSec, unsigned uMinMB, unsigned uMaxMB);

    BOOL DoDispose();
    
    BOOL CheckSpace();

private:
    void Construct();
    ULONGLONG BytesAllocated(ULONGLONG used) const;
    BOOL IsHappy() const;
    BOOL RecurseSpaceUsed(__inout_ecount(cchPathBuf) TCHAR *rwchPath, Size_t cchCurPath, const Size_t cchPathBuf, unsigned depth, ULONGLONG* pcchTotal) const;
    BOOL CalcSpaceUsed(const TCHAR* wszFolder, ULONGLONG* pcch) const;

    unsigned    m_uCheckPeriodSec;
    unsigned    m_uMinMB;
    unsigned    m_uMaxMB;
    time_t      m_tLastCheck;
    BOOL        m_fHappy;
    ULONGLONG   m_ulClusterSize;
    TCHAR       m_wszFolder[MAX_PATH];
#if !APR_HAVE_WINDOWS_H
    char        m_szFolder[MAX_PATH];
#endif
};
