/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "basic_types.h"
#include "SSPTypes.h"


// The function generate a signature for document based on document's title, anchor, anchor text and content.
// There is a small chance that signature can be same even when document are different (ie, hash collision)
// Also, since we use light weigh parse (no DOM tree generated), the parsing result might not be exact same as other html parsers
//
// The basic approach of the parser is to extract title string first from html head part.
// Then, from body part, go through every char from top to bottom and extract anchors in <a> tag, and anchor text and content strings between tags
Signature ComputeSignature(const char* pbDoc, Size_t cbDoc);


// Extract Canonical tag from http response for normalization purpose
// We treat Canonical url as an authoritative URL, so no post-processing on canonical url (ie, trim after # and etc)
// Return NULL and cbCanonicalLink = 0 when there is no canonical or we found the link is invalid
const char* ExtractCanonicalLink(const char* pbDoc, Size_t cbDoc, Size_t* cbCanonicalLink);

