/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#include "DocSignatureHash.h"

DocSignatureHash::DocSignatureHash()
    : m_currentHash(0)
    , m_numTokens(0)
{
    memset(m_tokenHashAggregation, 0, sizeof(m_tokenHashAggregation));
}

DocSignatureHash::~DocSignatureHash()
{
    m_currentHash = 0;
    m_numTokens = 0;
    memset(m_tokenHashAggregation, 0, sizeof(m_tokenHashAggregation));
}

void DocSignatureHash::AddTokenToLocation(Size_t nLocation)
{
    m_tokenHashAggregation[nLocation] = (m_tokenHashAggregation[nLocation] == 0) ? 1 : 0;
    
    m_numTokens++;
}

void DocSignatureHash::AddToken(Size_t tokenValue)
{
    AddTokenToLocation(tokenValue % SIZE_OF_SIGNATURE);
}

void DocSignatureHash::CalculateHash()
{
    UInt32 newHash = 0;
    for (int i = 0; i < SIZE_OF_SIGNATURE; i++)
    {

        // when i = 0, newHash=0 after newHash <<= 1
        newHash <<= 1;
        newHash |= m_tokenHashAggregation[i];
    }
    m_currentHash = newHash;
    memset(m_tokenHashAggregation, 0, sizeof(m_tokenHashAggregation));
}

Signature DocSignatureHash::GetHashValue() const
{
    return m_currentHash;
}

UInt32 DocSignatureHash::GetNumTokens() const
{
    return m_numTokens;
}

int DocSignatureHashCppLineCount()
{
    return __LINE__;
}