/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "basic_types.h"
#include "SSPTypes.h"

/*********************************************************************************************/
/* The class is not thread safe, but we assume only each thread has its own Hash object
/* The way this hash structure works is:
/* 1. Each token maps to one bit of this 32 bits hash
/* 2. When token is added, flip the corresponding bit ( 1->0 or 0-1 )
/*********************************************************************************************/
class DocSignatureHash
{
public:
    DocSignatureHash();
    ~DocSignatureHash();

    // Adds a token to certain bit for hash calculation.
    void AddTokenToLocation(Size_t nLocation);

    // Adds a token to hash calculation
    void AddToken(Size_t tokenValue);

    // Ends calculation of the current hash and makes it available for reading.
    void CalculateHash();
    // Gets a value of the current hash. Note that you should call CalculateSimHash method before you try to read it.
    Signature GetHashValue() const;
    // Gets number of tokens added to hash structure
    UInt32 GetNumTokens() const;

private:
    int m_tokenHashAggregation[SIZE_OF_SIGNATURE];
    Signature m_currentHash;
    UInt32 m_numTokens;
};