/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once
#include "basic_types.h"

//
// This method check if two strings are same or not
// true if they are same, false if they are different
//
bool IsMatch(const char* begin, const char* end, const char* str);

//
// This method is a wrapper of strlen(), and get # of chars in string
// \0 is excluded
//
Size_t GetSizeOfTag(const char* str);

//
// This method extract content between open tags and close tags 
//
const char* ExtractContent(const char* input, const char* end, Size_t* tokenLength, Size_t* charScanned, Size_t* charIgnored);
bool Extract(const char* input, const char* end, Size_t* startPos, Size_t* endPos, Size_t* ignoredCharsCount);

//
// Extract link from anchor
//
const char* ExtractAnchor(const char* input, const char* end, Size_t* tokenLength);

//
// Extract canonical link from head of html
//
const char* ExtractCanonicalLink(const char* input, const char* end, Size_t* tokenLength);
bool ExtractAttributeValue(const char* input, const char* end, Size_t* startPos, Size_t* endPos);

//
// Skip all chars up to specific string, return # of chars skipped 
//
Size_t SkipTo(const char* input, const char* end, const char* skipToTag);

//
// Skip all comments or scripts from document
//
Size_t SkipCommentOrScript(const char* input, const char* end);

bool IsSpace(char c);
bool IsPunct(char c);
bool IsAlNum(char c);
bool IsSkipAttributeChar(char c);
bool IsEndOfAttributeChar(char c);


static const char* TAG_CLOSE = ">";
static const char* TAG_SELF_CLOSE = "/>";
static const char* TAG_OPEN = "<";

static const char* TAG_TITLE_OPEN = "<title";
static const char* TAG_TITLE_CLOSE = "</title>";

static const char* TAG_HEAD_OPEN = "<head";
static const char* TAG_HEAD_CLOSE = "</head>";
static const char* TAG_BODY_OPEN = "<body";
static const char* TAG_BODY_CLOSE = "</body>";
static const char* TAG_ANCHOR_OPEN = "<a ";
static const char* TAG_LINK_OPEN = "<link ";
static const char* TAG_SCRIPT_OPEN = "<script";
static const char* TAG_SCRIPT_CLOSE = "</script>";
static const char* TAG_COMMENT_OPEN = "<!--";
static const char* TAG_COMMENT_CLOSE = "-->";

static const char* TAG_HREF= " href";
static const char* TAG_REL= " rel";



