/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "bloomfilter.h"

class DoubleBloomFilter
{
    BloomFilter m_filters[2];
    unsigned m_uDurationSec;
    unsigned m_cSlots;
    size_t m_cbSlotSize;
    volatile LONG m_lCurSlot;
    volatile LONG64 m_cNewSlotEntries;
    volatile time_t m_tLastReset;
    volatile UInt32 m_lock;

public:
    BOOL Initialize(unsigned uOneSlotDurationSec, // how often we flip between the 2 filters (e.g., 24 hours)
                    size_t cbOneSlotBytes, // size of each filter in bytes.  Total allocated memory is ~2x this.
                    IAllocator* pAlloc,
                    unsigned cSlots = 2);
//
// Returns TRUE if url was already present in either of the two slots.
// Otherwise returns FALSE and adds url to the current slot.
//
// Flipping of the current-vs-old slots happens automatically inside this function when needed.
// Flipping phase alignment is unspecified - implementation might use local time, GMT, or when the filter was created.
//
// The caller needs to call this periodically - with a dummy URL if necessary - to cause old entries to get flushed out.
// 
// This function tries to be thread-safe - it should not be necessary to have a lock around it.
//
    BOOL Check(size_t cbUrl, const char* pbUrl); // url - not necessarily NULL-terminated
    UInt64 Count();
};

