/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once
#include "PAL.h"
#include "SSPCommon.h"

#if APR_HAVE_WINDOWS_H

// Set ACE to the file ACL for a given group name
// Note: If there is ACE for the same game group name already exists, the function will replace it with the new access mask.
BOOL SetFileAccess(const TCHAR* wszFilePath, const char* szGroupName, DWORD dwAccessMask);

// Get SID of a given group name
BOOL GetAccountSID(const char* szGroupName, void** ppSid, DWORD* pcbSid = NULL);

// Get Security Descriptor from a given file name
BOOL GetFileSD(const TCHAR* wszFilePath, PSECURITY_DESCRIPTOR* pFileSD, const SECURITY_INFORMATION secInfo);

// Copy ACEs from the current DACL to new DACL object to modify ACL 
BOOL CopyACEs(const PACL pAcl, const PACL pNewAcl, const PSID pSid, DWORD aceCount, BOOL fInheritACEs = FALSE); 

// Copy the old security descriptor control flags regarding DACL automatic inheritance
// if SetSecurityDescriptorControl() API is available in advapi32.dll
BOOL CopyAutomaticInheritSecurityDescriptorControlFlags(const PSECURITY_DESCRIPTOR pFileSD, SECURITY_DESCRIPTOR* newSD);

#else

// For Linux port, we may not need every single function defined for Win32

BOOL SetFileAccess(const TCHAR* wszFilePath);

#endif
