/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "basic_types.h"

class HostMapping
{
public:

	class Entry
    {
		friend class HostMapping;

        // in
        const TCHAR* m_wszHost;
        const TCHAR* m_wszPort;
        const TCHAR* m_wszApplPath;

        // out
        const TCHAR* m_wszOutHostAndPort;		// If port number is not specified, it assumes 80.
        const TCHAR* m_wszOutApplPath;

        // linked list
        class Entry* m_pNext;

        BOOL Match(const TCHAR* wszHost, const TCHAR* wszPort, const TCHAR* wszApplPath);
    public:
        void* m_pUser;

        Entry() { memset(this, 0, sizeof(*this)); }
        
		Entry* NextEntry();

		// setters
		BOOL SetKey(IAllocator* pAlloc, const TCHAR* wszHost, const TCHAR* wszPort, const TCHAR* wszApplPath);
		BOOL SetValue(IAllocator* pAlloc, const TCHAR* wszOutHostAndPort, const TCHAR* wszOutApplPath);
		
		// getters
        BOOL GetHostAndPort(__out_ecount(cchOutHostAndPort) TCHAR* wszOutHostAndPort, Size_t cchOutHostAndPort);
        BOOL GetApplPath(__out_ecount(cchOutApplPath) TCHAR* wszOutApplPath, Size_t cchOutApplPath);

        BOOL CopyToSharedMemory(IAllocator* pAlloc, __out_bcount(cbDest) TCHAR** wszDest, Size_t cbDest, const TCHAR* wszSrc);


    } *m_pEntries;

    HostMapping() : m_pEntries(NULL) { };
    BOOL Initialize(IAllocator* pAlloc);
	BOOL Load(IAllocator* pAlloc, const TCHAR* wszMappingFile);
    Entry* Map(const TCHAR* wszHost, const TCHAR* wszPort, const TCHAR* wszApplPath);
    BOOL Enumerate(void* pvContext, BOOL (*pfnCallback)(void* pvContext, void* pUser));

private:
	BOOL AddEntry(IAllocator* pAlloc, const TCHAR* wszKey, const TCHAR* wszValue);
    const TCHAR* Extract(__out_ecount(cchExtracted) TCHAR* wszExtracted, Size_t cchExtracted, const TCHAR* wszCombined, const TCHAR* delim);
    BOOL ExtractElements(__out_ecount(cchHost) TCHAR* wszHost, Size_t cchHost, 
                                         __out_ecount(cchPort) TCHAR* wszPort, Size_t cchPort,
                                         TCHAR** wszApplPath, 
                                         const TCHAR* wszCombined);
};

