/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#include "PAL.h"

PAL_Parameters g_PALParams = {0};

BOOL PAL_Initialize(BOOL bPreIntialize)
{
    bPreIntialize;
    BOOL bSuccess = TRUE;

    if (g_PALParams.Version != 0)
    {
        // Already initialized.
        bSuccess = FALSE;
        goto Done;
    }

#ifdef __TARGET_APACHE__
    apr_status_t status;

    // APR module will normally call this function.
    if (bPreIntialize == TRUE)
    {
        status = apr_initialize();
        if (status != APR_SUCCESS)
        {
            bSuccess = FALSE;
            goto Done;
        }

        status = apr_pool_initialize();
        if (status != APR_SUCCESS)
        {
            apr_terminate(); 
            bSuccess = FALSE;
            goto Done;
        }

        g_PALParams.bPreInit = TRUE;
    }
    
    status = apr_pool_create(&g_PALParams.pMemPool, NULL);
    if (status != APR_SUCCESS)
    {
        bSuccess = FALSE;
        goto Done;
    }

    // APR module will normally call this function.
    if (bPreIntialize)
    {
        status = apr_atomic_init(g_PALParams.pMemPool);
        if (status != APR_SUCCESS)
        {
            bSuccess = FALSE;
            goto Done;
        }
    }
#endif

    // Set this when everything is successful.
    g_PALParams.Version = 1;

Done:
    if (g_PALParams.Version == 0)
    {
        // Cleanup if there is failure
        PAL_Terminate();
    }

    return bSuccess;
}


void PAL_Terminate(void)
{
#ifdef __TARGET_APACHE__
    if (g_PALParams.pMemPool != NULL)
    {
        apr_pool_destroy(g_PALParams.pMemPool);
    }
    
    if (g_PALParams.bPreInit == TRUE)
    {
        apr_pool_terminate();
        apr_terminate(); 
    }
#endif

    memset(&g_PALParams, 0, sizeof(g_PALParams));
}

