/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "basic_types.h"

#define FGETWS(a, b, c) _fgetts((a), (b), (c))

class PALFile
{
public:
    static BOOL s_fFakeGeneratedFileTimes; // used in test mode

    // Moves file (and overwrites existing) from ptszSrc to ptszDest.
    // Returns TRUE on success or FALSE for failure.
    static BOOL Move(const TCHAR *ptszSrc, const TCHAR *ptszDest);

    // Delete file ptszFile
    // Returns TRUE on success or FALSE for failure.
    static BOOL Delete(const TCHAR *ptszFile);

    // Opens a file.
    // Returns 0 for success; otherwise returns non-zero for failure.
    static int Open(FILE **ppFile, const TCHAR *ptszFilename, const TCHAR *ptszMode);

    // Sets the file's Last Modified time to now
    // "now" has special behavior in test mode.  Does that make this a bad place for the function ?
    static void SetModifiedTime(const TCHAR* wszFile);
};

class PALDirectory
{
public:
    // Returns TRUE on success or FALSE for failure.
    static BOOL CreateDirectory(const TCHAR *ptszFolder);

    // Delete folder ptszFile if empty.
    // Returns TRUE on success or FALSE for failure.
    static BOOL DeleteDirectory(const TCHAR *ptszFolder);

    // Creates a directory if it doesn't exist.
    // It will also create parent directories if they don't
    // exists.
    // Returns TRUE on success or FALSE for failure.
    static BOOL EnsureDirectoryExists(const TCHAR* wszPath);

    // Checks whether path exists.
    // Returns TRUE if this is a folder OR file.
    static BOOL PathExists(const TCHAR *ptszPath);

    // Retrieve the current working directory of the calling process.
    // Return TRUE if it was succuessful.
    static BOOL GetWorkingDirectory(__out_ecount(len) TCHAR *ptszPath, Size_t len);
};

class PALFind
{
public:
    enum EAttrib
    {
        File,
        Directory,
        Unknown
    };

    enum EStatus
    {
        Success,
        NoMoreResults,
        Error_FileNotFound,
        Error_DirectoryNotFound,
        Error_NotInitialized,
        Error_Other
    };

    PALFind();

    ~PALFind();

    BOOL Initialize(const TCHAR *ptszPath, const TCHAR *ptszSearchPattern);

    void Destroy();

    EStatus Find();

    EAttrib GetAttrib();

    const TCHAR *GetName();

    UInt64 GetSize();

private:
    PALFind(const PALFind &);
    const PALFind &operator=(const PALFind &);
    
#if APR_HAVE_WINDOWS_H
    EStatus LastError();

    bool            m_bInit;
    HANDLE          m_hFind;
    WIN32_FIND_DATA m_FindFileData;
    TCHAR           m_ptszSearchPath[MAX_PATH];

#else
    bool         m_bInit;
    apr_pool_t  *m_pPool;
    apr_dir_t   *m_pFolder; 
    apr_finfo_t  m_finfo;
    char         m_szPath[MAX_PATH];
    char         m_szPattern[MAX_PATH];
    TCHAR        m_tszName[MAX_PATH];
#endif
};
