/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#include "PALString.h"
#ifndef _UNICODE
#include "errno.h"
#endif

int PALString::Copy(__out_ecount(DestLen) char *pszDest, Size_t DestLen, const char *pszSrc)
{
	return Copy(pszDest, DestLen, pszSrc, strlen(pszSrc) + 1);
}

#ifdef _UNICODE
int PALString::Copy(__out_ecount(DestLen) TCHAR *ptszDest, Size_t DestLen, const TCHAR *ptszSrc)
{
	return Copy(ptszDest, DestLen, ptszSrc, TSTRLEN(ptszSrc) + 1);
}
#endif

int PALString::Copy(__out_ecount(DestLen) char *pszDest, Size_t DestLen, const char *pszSrc, Size_t srcLen)
{
#if APR_HAVE_WINDOWS_H
    return strncpy_s(pszDest, DestLen, pszSrc, srcLen);
#else
    // Not super efficient.
    if (pszSrc == NULL ||
        pszDest == NULL ||
        DestLen <= 0)
    {
        return EINVAL;
    }

    if (srcLen > DestLen)
    {
        return EINVAL;
    }

    strncpy(pszDest, pszSrc, srcLen);
    pszDest[DestLen - 1] = 0;

    return 0;
#endif
}

#ifdef _UNICODE
int PALString::Copy(__out_ecount(DestLen) TCHAR *ptszDest, Size_t DestLen, const TCHAR *ptszSrc, Size_t srcLen)
{
#if APR_HAVE_WINDOWS_H
    return _tcsncpy_s(ptszDest, DestLen, ptszSrc, srcLen);
#else
    // Not super efficient.
    if (ptszSrc == NULL ||
        ptszDest == NULL ||
        DestLen <= 0)
    {
        return EINVAL;
    }

    if (srcLen > DestLen)
    {
        return EINVAL;
    }

    wcsncpy(ptszDest, ptszSrc, srcLen);
    ptszDest[DestLen - 1] = 0;

    return 0;
#endif
}
#endif

int PALString::Convert(__out_ecount(DestLen) TCHAR *ptszDest, Size_t DestLen, const char *pszSrc, Size_t SrcLen)
{
#ifdef _UNICODE
#if APR_HAVE_WINDOWS_H
    Size_t len;
    return mbstowcs_s(&len, ptszDest, DestLen, pszSrc, SrcLen + 1);
#else
    if (pszSrc == NULL ||
        ptszDest == NULL ||
        DestLen <= 0 ||
        SrcLen > DestLen - 1)
    {
        return EINVAL;
    }

    if (mbstowcs(ptszDest, pszSrc, SrcLen + 1) >= 0)
    {
        return 0;
    }

    return EINVAL;
#endif

#else
    if (pszSrc == NULL ||
        ptszDest == NULL ||
        DestLen <= 0 ||
        SrcLen > DestLen - 1)
    {
        return EINVAL;
    }

    memcpy(ptszDest, pszSrc, (SrcLen + 1) * sizeof(char));
    return 0;
#endif
}

#ifdef _UNICODE
int PALString::Convert(__out_ecount(DestLen) char *pszDest, Size_t DestLen, const TCHAR *ptszSrc, Size_t SrcLen)
{
#if APR_HAVE_WINDOWS_H
    Size_t len;
    return wcstombs_s(&len, pszDest, DestLen, ptszSrc, SrcLen + 1);
#else
    if (ptszSrc == NULL ||
        pszDest == NULL ||
        DestLen <= 0 ||
        SrcLen > DestLen - 1)
    {
        return EINVAL;
    }

    if (wcstombs(pszDest, ptszSrc, SrcLen + 1) >= 0)
    {
        return 0;
    }

    return EINVAL;
#endif
}
#endif

int PALString::Compare(const TCHAR *ptszStr1, const TCHAR *ptszStr2)
{
#if APR_HAVE_WINDOWS_H
    return _tcscmp(ptszStr1, ptszStr2);
#else
#ifdef _UNICODE
    return wcscmp(ptszStr1, ptszStr2);
#else
    return strcmp(ptszStr1, ptszStr2);
#endif
#endif
}

int PALString::CompareInsensitive(const TCHAR *ptszStr1, const TCHAR *ptszStr2)
{
#if APR_HAVE_WINDOWS_H
    return _tcsicmp(ptszStr1, ptszStr2);
#else
#ifdef _UNICODE
    return wcsicmp(ptszStr1, ptszStr2);
#else
    return strcasecmp(ptszStr1, ptszStr2);
#endif
#endif
}

int PALString::CompareInsensitive(const char *string1, const char *string2, Size_t count)
{
#if APR_HAVE_WINDOWS_H
    return _memicmp(string1, string2, count);
#else
    return strncasecmp(string1, string2, count);
#endif
}
