/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "PALDefs.h"
#include "basic_types.h"

#if APR_HAVE_WINDOWS_H
// Windows Platform
#define TSTRLEN(a) _tcslen((a))
#define PRINTF_S(...) printf_s(__VA_ARGS__)
#define TPRINTF_S(...) _tprintf_s(__VA_ARGS__)
#define SPRINTF_S(a,b,c,...) sprintf_s((a),(b),(c),__VA_ARGS__)
#define SWPRINTF_S(a,b,c,...) swprintf_s((a),(b),(c),__VA_ARGS__)
#define STPRINTF_S(a,b,c,...) _stprintf_s((a),(b),(c),__VA_ARGS__)
#define SNPRINTF(a,b,c,...) _snprintf((a),(b),(c),__VA_ARGS__)
#define SNPRINTF_S(a,b,c,...) _snprintf_s((a),(b),(c),__VA_ARGS__)
#define SNTPRINTF(a,b,c,...) _sntprintf((a),(b),(c),__VA_ARGS__)
#define SNTPRINTF_S(a,b,c,d,...) _sntprintf_s((a),(b),(c),(d),__VA_ARGS__)
#define VSPRINTF_S(a,b,c,d) vsprintf_s((a),(b),(c),(d))
#define VSNPRINTF(a,b,c,d) _vsnprintf((a),(b),(c),(d))
#define VSNPRINTF_S(a,b,c,d,e) _vsnprintf_s((a),(b),(c),(d),(e))
#define SSCANF_S(a,b,...) sscanf_s((a),(b),__VA_ARGS__)
#define STSCANF_S(a,b,...) _stscanf_s((a),(b),__VA_ARGS__)
#define TCSSTR(a,b) _tcsstr((a),(b))
#define STRCAT_S(dst,size,src) strcat_s(dst,size,src)

#define PathStringCompare _tcsicmp

#define CHAR_DIRSLASH '\\'
#define STRFMT_DIRSLASH "\\"
#define STRFMT_TDIRSLASH TEXT("\\")
#define STRFMT_I32 "%I32"
#define STRFMT_I64 "%I64"

#else
// Linux Platform
// TODO: Note, all the "secured" versions only use the non-secured version
// and may result in slight behavior change. Later, we should implement
// secured equivalent.
#ifdef _UNICODE
#define TSTRLEN(a) wcslen((a))
#define TPRINTF_S(...) wprintf(__VA_ARGS__)
#define STPRINTF_S(a,b,c,...) swprintf((a),(c),__VA_ARGS__)
#define SNTPRINTF(a,b,c,...) swprintf((a),(b),(c),__VA_ARGS__)
#define SNTPRINTF_S(a,b,c,d,...) swprintf((a),(b),(d),__VA_ARGS__)
#define TCSSTR(a,b) wcsstr((a),(b))
#define PathStringCompare wcscmp
#else
#define TSTRLEN(a) strlen((a))
#define TPRINTF_S(...) printf(__VA_ARGS__)
#define STPRINTF_S(a,b,c,...) sprintf((a),(c),__VA_ARGS__)
#define SNTPRINTF(a,b,c,...) snprintf((a),(b),(c),__VA_ARGS__)
#define SNTPRINTF_S(a,b,c,d,...) snprintf((a),(b),(d),__VA_ARGS__)
#define PathStringCompare strcmp
#endif

#define PRINTF_S(...) printf(__VA_ARGS__)
#define SPRINTF_S(a,b,c,...) snprintf((a),(b),(c),__VA_ARGS__)
#define SWPRINTF_S(a,b,c,...) swprintf((a),(b),(c),__VA_ARGS__)
#define SNPRINTF(a,b,c,...) snprintf((a),(b),(c),__VA_ARGS__)
#define SNPRINTF_S(a,b,c,...) snprintf((a),(b),__VA_ARGS__)
#define VSPRINTF_S(a,b,c,d) vsprintf((a),(c),(d))
#define VSNPRINTF(a,b,c,d) vsnprintf((a),(b),(c),(d))
#define VSNPRINTF_S(a,b,c,d,e) vsnprintf((a),(b),(d),(e))
#define SSCANF_S(a,b,...) sscanf((a),(b),__VA_ARGS__)
#define STSCANF_S(a,b,...) sscanf((a),(b),__VA_ARGS__)
#define TCSSTR(a,b) strstr((a),(b))
#define STRCAT_S(dst,size,src) (strcat(dst,src),0) // simulate 0 errno returned by strcat_s


#define CHAR_DIRSLASH '/'
#define STRFMT_DIRSLASH "/"
#define STRFMT_TDIRSLASH TEXT("/")
#define STRFMT_I32 "%"

#ifdef __LP64__
#define STRFMT_I64 "%l"
#else
#define STRFMT_I64 "%ll"
#endif

#endif


#ifdef _UNICODE
#define STRFMT_LS "%ls"
#define STRFMT_TLS TEXT("%ls")
#define STRFMT_THS TEXT("%hs")
#define STRFMT_TS "%ls"
#define STRFMT_NTLS TEXT("%.*ls")
#define _fgetts fgetws
#else
#define STRFMT_LS "%ls"
#define STRFMT_TLS TEXT("%s")
#define STRFMT_THS TEXT("%s")
#define STRFMT_TS "%s"
#define STRFMT_NTLS TEXT("%.*s")
#define _fgetts fgets
#endif


#if !defined(_TRUNCATE)
#define _TRUNCATE ((Size_t)-1)
#endif


class PALString
{
public:
    // pszSrc is a null-terminated string.
    // If successful, pszDest is a null-terminated string. 
    // Returns 0 if successful.
    static int Copy(__out_ecount(DestLen) char *pszDest, Size_t DestLen, const char *pszSrc);

#ifdef _UNICODE
    static int Copy(__out_ecount(DestLen) TCHAR *ptszDest, Size_t DestLen, const TCHAR *ptszSrc);
#endif

    // For partial copy, DestLen should be at least SrcLen + 1
	static int Copy(__out_ecount(DestLen) char *pszDest, Size_t DestLen, const char *pszSrc, Size_t SrcLen);

#ifdef _UNICODE
    static int Copy(__out_ecount(DestLen) TCHAR *ptszDest, Size_t DestLen, const TCHAR *ptszSrc, Size_t SrcLen);
#endif

    // pszSrc is a null-terminated string & SrcLen is the length of pszSrc not including
    // null-terminated.
    // Returns 0 if successful.
    static int Convert(__out_ecount(DestLen) TCHAR *ptszDest, Size_t DestLen, const char *pszSrc, Size_t SrcLen);

#ifdef _UNICODE
    static int Convert(__out_ecount(DestLen) char *ptszDest, Size_t DestLen, const TCHAR *ptszSrc, Size_t SrcLen);
#endif

    static int Compare(const TCHAR *ptszStr1, const TCHAR *ptszStr2);

	static int CompareInsensitive(const TCHAR *ptszStr1, const TCHAR *ptszStr2);

    static int CompareInsensitive(const char *string1, const char *string2, Size_t count);
};

