/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "basic_types.h"

class PALInterlocked
{
public:
    // Returns the resulting incremented value.
    static UInt32 Increment(volatile UInt32 *pMem);

    // Returns the resulting decremented value.
    static UInt32 Decrement(volatile UInt32 *pMem);

    // Returns the initial value of pMem.
    static UInt32 Add(volatile UInt32 *pMem, UInt32 Value);

    static void Set(volatile UInt32 *pMem, UInt32 Value);

    // Returns the initial value of pMem.
    static UInt32 CompareSet(volatile UInt32 *pMem, UInt32 Value,  UInt32 Comparand);
};

class PALEvent
{
public:
    enum EWaitState
    {
        Signaled,
        Timeout,
        Failed
    };

    PALEvent();

    ~PALEvent();

    BOOL Initialize();

    void Deinitialize();

    PALEvent::EWaitState Wait(DWORD dwMilliSeconds);

    BOOL Signal();

private:
    // No copying/assignment allowed.
    PALEvent(const PALEvent &);
    PALEvent &operator=(const PALEvent &);

#ifdef __TARGET_IIS__
    HANDLE  m_hEvent;
#else
    apr_pool_t         *m_pPool;
    apr_thread_mutex_t *m_pMutex;
    apr_thread_cond_t  *m_pCondVar;
#endif
};
