/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once
#include "PAL.h"
#include "basic_types.h"

// Vista threadpool theoretically protects IIS from this plugin by giving
// the plugin its own threadpool, separate from the process threadpool.
#if (_WIN32_WINNT >= 0x0600)
#define __VISTA_THREADPOOL__
#endif

class PALThread
{
public:
    PALThread();

    ~PALThread();

    BOOL Initialize(void *pPlatformMemPool = NULL);

    typedef void(*PFN)(void*);
    static PFN s_pfn; // Limited to 1 PALThread client to avoid having to malloc()/free() a redundant struct for every work item
    static void StaticSetCallbackFunction(PFN);

    BOOL Deinitialize();

    BOOL DispatchWork(void *pContext);

private:
#ifdef __TARGET_IIS__
#ifdef __VISTA_THREADPOOL__
    PTP_POOL                m_threadPool;
    TP_CALLBACK_ENVIRON     m_callbackEnv;
#endif
#else
    apr_thread_pool_t      *m_pThreadPool;
    apr_pool_t             *m_pPool;
#endif
};
