/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#include "SSPCommon.h"
#include "SSPConfig.h"
#include "PALString.h"
#include "PALFile.h"

#define WEBSERVER TEXT("SitemapPlugin")
#define STR_FOLDER_SITEMAPPLUGIN STRFMT_TDIRSLASH TEXT("Microsoft") STRFMT_TDIRSLASH WEBSERVER
#define LINUX_DATA_FOLDER TEXT("/var/local")

BOOL MyFileReader::Initialize(const TCHAR* wszFile, const unsigned *pdwExpectedMagic)
{
    BOOL fRet = FALSE;
    DWORD dwMagic = 0, cbRead = 0;

    if (NOERROR != m_reader.DoInit(wszFile)) goto Exit;
    if (NOERROR != m_reader.InitializeUncompression()) goto Exit; // this is likely an out of memory issue
    if ((NOERROR != m_reader.GetData(&dwMagic, 4, &cbRead) != NO_ERROR) || (cbRead != 4)) goto Exit;
    for (m_version = 0; *pdwExpectedMagic; m_version++, pdwExpectedMagic++)
    {
        if (*pdwExpectedMagic == dwMagic)
        {
            fRet = TRUE;
            break;
        }
    }
Exit:
    return fRet;
}

bool MyFileReader::Read(__out_bcount(cb) void* pv, Size_t cb)
{
    DWORD cbRead = 0;
    return (NOERROR == m_reader.GetData(pv, (DWORD)cb, &cbRead)) && (cbRead == cb);
}

static TCHAR g_wszDataFolder[MAX_PATH] = TEXT("");

BOOL SetDataFolder(const TCHAR* wszDataFolder)
{
    return PALString::Copy(g_wszDataFolder, ARRAYSIZE(g_wszDataFolder), wszDataFolder) == 0 ?
TRUE : FALSE;
}


#ifdef __TARGET_IIS__
static BOOL PluginConfQueryDataFolder(__out_ecount(cchFolder) TCHAR* wszFolder, Size_t cchFolder)
{   
    if (!PluginRegQueryValue(NULL, SSP_REGISTRY_DATAFOLDERPATH, wszFolder, cchFolder)) return FALSE;
    return TRUE;
}

BOOL GetDataFolder(__out_ecount(cchFolder) TCHAR* wszFolder, Size_t cchFolder)
{
    if (g_wszDataFolder[0])
    {
        return PALString::Copy(wszFolder, cchFolder, g_wszDataFolder) == 0 ? TRUE :FALSE;
    }

    if (PluginConfQueryDataFolder(wszFolder, cchFolder))
    {
        return TRUE;
    }

    TCHAR wszCommonFolder[MAX_PATH];
    if (NOERROR != SHGetFolderPath(NULL, CSIDL_COMMON_APPDATA, NULL, 0, wszCommonFolder))
    {
        return FALSE;
    }
    if (-1 == SNTPRINTF_S(wszFolder, cchFolder, cchFolder - 1, STRFMT_TLS STRFMT_TLS, wszCommonFolder, STR_FOLDER_SITEMAPPLUGIN))
    {
        return FALSE;
    }
    return TRUE;
}
#else
BOOL GetDataFolder(TCHAR* wszFolder, Size_t cchFolder)
{
    BOOL bRetValue = TRUE;

    if (g_wszDataFolder[0])
    {
        bRetValue = PALString::Copy(wszFolder, cchFolder, g_wszDataFolder) == 0 ? TRUE : FALSE;
        goto Done;
    }

    if (-1 == SNTPRINTF_S(wszFolder, cchFolder, cchFolder - 1,
        STRFMT_TLS STRFMT_TLS,
        LINUX_DATA_FOLDER, STR_FOLDER_SITEMAPPLUGIN))
    {
        bRetValue = FALSE;
        goto Done;
    }

Done:
    return bRetValue;
}
#endif

BOOL GetDataFolderWithIndex(const TCHAR* wszDataFolderPrefix, unsigned index, __out_ecount(cchOut) TCHAR* wszOut, Size_t cchOut)
{
    return (0 < STPRINTF_S(wszOut, cchOut, STRFMT_TLS TEXT("_%u"), wszDataFolderPrefix, index));
}

BOOL GetFullPath(const TCHAR* wszFolder, const TCHAR* wszFilename, __out_ecount(cchFullPath) TCHAR* wszFullPath, Size_t cchFullPath)
{
    Size_t cchFilename = TSTRLEN(wszFilename);
    Size_t cchFolder = TSTRLEN(wszFolder);
    while (cchFolder > 0 && ((wszFolder[cchFolder-1] == '\\') || (wszFolder[cchFolder-1] == '/'))) cchFolder--;

    if (cchFolder + 1 + cchFilename >= cchFullPath)
    {
        return FALSE;
    }
    memcpy(wszFullPath, wszFolder, cchFolder * sizeof(TCHAR));

    wszFullPath[cchFolder] = CHAR_DIRSLASH;

    memcpy(wszFullPath + cchFolder + 1, wszFilename, (cchFilename + 1) * sizeof(TCHAR));
    return TRUE;
}

BOOL GetConfigFile(const TCHAR* wszFolder, __out_ecount(cchConfigFile) TCHAR* wszConfigFile, Size_t cchConfigFile)
{
    return GetFullPath(wszFolder, TEXT("config.ini"), wszConfigFile, cchConfigFile);
}

BOOL GetCacheFile(const TCHAR* wszFolder, __out_ecount(cchCacheFile) TCHAR* wszCacheFile, Size_t cchCacheFile)
{
    return GetFullPath(wszFolder, TEXT("cache.bin"), wszCacheFile, cchCacheFile);
}

BOOL GetPluginCountersFile(const TCHAR* wszFolder, __out_ecount(cchCacheFile) TCHAR* wszCacheFile, Size_t cchCacheFile)
{
    return GetFullPath(wszFolder, TEXT("PluginCounters.tsv"), wszCacheFile, cchCacheFile);
}

BOOL GetAllTimeCountersFile(const TCHAR* wszFolder, __out_ecount(cchFile) TCHAR* wszFile, Size_t cchFile)
{
    return GetFullPath(wszFolder, TEXT("AllTimeCounters.tsv"), wszFile, cchFile);
}

BOOL NullTerminateString(__out_ecount(cchString) TCHAR* wszString, Size_t cchString, Size_t cchEnd)
{
    BOOL fRet = FALSE;
    if ('\0' != wszString[cchString-1]) 
    {
        if (cchEnd >= cchString) goto Exit; // no room to add '\0'
        wszString[cchEnd] = '\0';
    }

    fRet = TRUE;

Exit:
    return fRet;
}

#ifdef __TARGET_IIS__
static BOOL RegQueryValueHelper(const TCHAR* wszSubkey, const TCHAR* wszValueName, __out_bcount_part(*pdwSize, *pdwSize) BYTE* pb, DWORD* pdwSize, DWORD* pdwType)
{
    TCHAR *wszKeyPath;
    TCHAR wszTemp[MAX_PATH];
    HKEY hKey = NULL;
    BOOL fRet = FALSE;

    if (wszSubkey && wszSubkey[0])
    {
        if (0 >= SNTPRINTF_S(wszTemp, ARRAYSIZE(wszTemp), STRFMT_TLS TEXT("\\") STRFMT_TLS, SSP_REGISTRY_PATH, wszSubkey)) return FALSE;
        wszKeyPath = wszTemp;
    }
    else
    {
        wszKeyPath = SSP_REGISTRY_PATH;
    }
    if (ERROR_SUCCESS != RegOpenKeyEx(SSP_REGISTRY_HIVE, wszKeyPath, 0, KEY_QUERY_VALUE, &hKey)) goto Exit;
    if (ERROR_SUCCESS != RegQueryValueEx(hKey, wszValueName, NULL, pdwType, pb, pdwSize)) goto Exit; // BUGBUG - ignoring the registry setting if we get ERROR_MORE_DATA
    fRet = TRUE;
Exit:
    if (hKey) RegCloseKey(hKey);
    return fRet;
}

BOOL PluginRegQueryValue(const TCHAR* wszSubkey, const TCHAR* wszValueName, __out_ecount(cchValue) TCHAR* wszValue, Size_t cchValue)
{
    DWORD dwType;
    DWORD cbBuf = (DWORD)cchValue * sizeof(TCHAR);
    BOOL fRet = FALSE;

    if (!RegQueryValueHelper(wszSubkey, wszValueName, (BYTE*)wszValue, &cbBuf, &dwType)) goto Exit;
    if (REG_SZ != dwType) goto Exit;
    if (0 != (cbBuf % sizeof(TCHAR))) goto Exit;
    size_t cch = cbBuf / sizeof(TCHAR);
    if (cch <= 0) goto Exit;
    if (!NullTerminateString(wszValue, cchValue, cch)) // string from registry not guaranteed to be NULL-terminated, so do it ourselves
    {
        goto Exit;
    }

    fRet = TRUE;
Exit:
    return fRet;
}

BOOL PluginRegQueryDWORD(const TCHAR* wszSubkey, const TCHAR* wszValueName, DWORD* pdw)
{
    DWORD dwType;
    DWORD cbBuf = sizeof(DWORD);

    if (!RegQueryValueHelper(wszSubkey, wszValueName, (BYTE*)pdw, &cbBuf, &dwType)) return FALSE;
    if ((REG_DWORD != dwType) || (sizeof(DWORD) != cbBuf)) return FALSE;

    return TRUE;
}

static BOOL PluginConfQuerySiteFolder(const TCHAR* wszHostname, const TCHAR* wszPort, __out_ecount(cchFolder) TCHAR* wszFolder, Size_t cchFolder)
{   
    TCHAR wszKey[MAX_PATH];
    if (0 >= SNTPRINTF_S(wszKey, ARRAYSIZE(wszKey), STRFMT_TLS TEXT("_") STRFMT_TLS, wszHostname, wszPort)) return FALSE;
    if (!PluginRegQueryValue(wszKey, SSP_REGISTRY_FOLDERPATH, wszFolder, cchFolder)) return FALSE;
    return TRUE;
}
#else

static BOOL PluginConfQuerySiteFolder(const TCHAR* wszHostname, const TCHAR* wszPort, __out_ecount(cchFolder) TCHAR* wszFolder, Size_t cchFolder)
{   
    // Host Mapping should be able to replace Registry-based site folder mapping so this feature won't be ported to Linux
    return FALSE; 
}

#endif

#if defined(__TARGET_APACHE__) || defined(__PUBLISH_MODE_MANUAL__)
static DWORD g_dwSiteMapPublishMode = SitemapPublishFull;
static DWORD g_dwSiteMapPingBackMode = 1;
static DWORD g_dwSiteMapHelpSSPMode = 1;

BOOL SetSiteMapPublishMode(DWORD mode)
{
    if (mode >= SitemapDontPublish && mode <= SitemapPublishFull)
    {
        if(mode != SitemapDontPublish)
        {
            g_dwSiteMapPublishMode = SitemapPublishFull;
        }
        else
        {
            g_dwSiteMapPublishMode = mode;
        }

        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

DWORD GetSiteMapPublishMode()
{
    return g_dwSiteMapPublishMode;
}


BOOL SetSiteMapPingBackMode(DWORD mode)
{
    if (mode == 0 || mode == 1)
    {
        g_dwSiteMapPingBackMode = mode;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

DWORD GetSiteMapPingBackMode()
{
    return g_dwSiteMapPingBackMode;
}

BOOL SetSiteMapHelpSSPMode(DWORD mode)
{
    if (mode == 0 || mode == 1)
    {
        g_dwSiteMapHelpSSPMode = mode;
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

DWORD GetSiteMapHelpSSPMode()
{
    return g_dwSiteMapHelpSSPMode;
}

#endif


// Construct site subfolder based on hostname and port number
BOOL GetSiteFolderPrefixByHostScheme(const TCHAR* wszHostname, const TCHAR* wszPort, __out_ecount(cchFolder) TCHAR* wszFolder, Size_t cchFolder)
{
    if (PluginConfQuerySiteFolder(wszHostname, wszPort, wszFolder, cchFolder))
    {
        return TRUE;
    }

    TCHAR wszParentFolder[MAX_PATH];
    if (!GetDataFolder(wszParentFolder, ARRAYSIZE(wszParentFolder)))
    {
        return FALSE;
    }

    int returnedBufferLength = 0;

    returnedBufferLength = SNTPRINTF_S(wszFolder, cchFolder, cchFolder - 1, STRFMT_TLS STRFMT_TDIRSLASH TEXT("site_") STRFMT_TLS TEXT("_") STRFMT_TLS, wszParentFolder, wszHostname, wszPort);

    if(returnedBufferLength == -1)
    {
        return FALSE;
    }

    return TRUE;
}

static TCHAR g_wszBaseIPCName[MAX_PATH] = TEXT("");

BOOL SetBaseIPCName(const TCHAR* wszName)
{
    return 0 == PALString::Copy(g_wszBaseIPCName, ARRAYSIZE(g_wszBaseIPCName), wszName);
}

BOOL GetIPCName(__out_ecount(cchFullName) TCHAR* wszFullName, Size_t cchFullName, const TCHAR* wszName, unsigned index)
{
    return 0 < STPRINTF_S(wszFullName, cchFullName, WEBSERVER TEXT("_") STRFMT_TLS TEXT("_") STRFMT_TLS TEXT("_%u"), g_wszBaseIPCName, wszName, index);
}

#if APR_HAVE_WINDOWS_H
#define TFOPENTEXT TEXT("rt, ccs=UNICODE")
#else
#define TFOPENTEXT TEXT("rt")
#endif

const TCHAR* IPCNameFromAppPoolName(const TCHAR* tszAppPoolName)
{
    const TCHAR* tszRet = tszAppPoolName;
    static TCHAR tszRetBuffer[MAX_PATH];
    TCHAR tszFolder[MAX_PATH], tszFile[MAX_PATH];
    TCHAR tszLine[1024];
    FILE* fp = NULL;
    Size_t cchAppPoolName = TSTRLEN(tszAppPoolName);

    if (!GetDataFolder(tszFolder, ARRAYSIZE(tszFolder)))
    {
        goto Exit;
    }
    if (!GetFullPath(tszFolder, TEXT("AppPoolMapping.txt"), tszFile, ARRAYSIZE(tszFile)))
    {
        goto Exit;
    }
    if (0 != PALFile::Open(&fp, tszFile, TFOPENTEXT))
    {
        goto Exit;
    }

    while (NULL != _fgetts(tszLine, ARRAYSIZE(tszLine), fp))
    {
        TCHAR* pcEq = NULL;
        for (TCHAR* p = tszLine; *p != '\0'; p++)
        {
            if ('=' == *p)
            {
                pcEq = p;
            }
            if (*p == '\r' || *p == '\n')
            {
                *p = '\0';
            }
        }
        
        if (!pcEq)
        {
            continue;
        }
        if (cchAppPoolName + tszLine != pcEq) // no whitespace please
        {
            continue;
        }
        if (memcmp(tszAppPoolName, tszLine, sizeof(TCHAR) * cchAppPoolName))
        {
            continue;
        }
        if (0 != PALString::Copy(tszRetBuffer, ARRAYSIZE(tszRetBuffer), pcEq + 1))
        {
            continue;
        }

        tszRet = tszRetBuffer; // this is only called once at initialiation, so I hope does not need to be thread safe
        break;
    }

    fclose(fp);

Exit:
    return tszRet;
}

BOOL CombineHostAndPort(__out_ecount(cchHostAndPort) TCHAR* wszHostAndPort, Size_t cchHostAndPort, const TCHAR* wszHost, const TCHAR* wszPort)
{
    TCHAR wszPortWithColon[20] = TEXT("");
    if (PALString::Compare(wszPort, TEXT("80")))
    {
        if (0 >= SNTPRINTF_S(wszPortWithColon, ARRAYSIZE(wszPortWithColon), _TRUNCATE, TEXT(":") STRFMT_TLS, wszPort))
        {
            return FALSE;
        }
    }
    return 0 < SNTPRINTF_S(wszHostAndPort, cchHostAndPort, cchHostAndPort - 1, STRFMT_TLS STRFMT_TLS, wszHost, wszPortWithColon);
}

static time_t g_tTestTime = 0;
void SetTestTime(time_t t)
{
    g_tTestTime = t;
}

time_t PluginTime()
{
    if (g_tTestTime)
    {
        return g_tTestTime;
    }
    else
    {
        return time(NULL);
    }
}

void PluginGetLocalTime(struct tm* ptm)
{
    time_t t = PluginTime();
    localtime_s(ptm, &t);
}
void PluginGetUTCTime(struct tm* ptm)
{
    time_t t = PluginTime();
    gmtime_s(ptm, &t);
}

int SSPCommonCppLineCount()
{
    return __LINE__;
}

int PluginVersionNumber()
{ // update this to reflect changes in files that don't have a LineCount() because they don't go into service exe (e.g. PluginLib)
    return 1;
}
