/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once
#include "PAL.h"
#include "FilesCommon.h"
#include "UncompressingReader.h"

BOOL SetDataFolder(const TCHAR* wszDataFolder);
BOOL GetDataFolder(__out_ecount(cchFolder) TCHAR* wszFolder, Size_t cchFolder);
BOOL GetDataFolderWithIndex(const TCHAR* wszDataFolderPrefix, unsigned index, __out_ecount(cchOut) TCHAR* wszOut, Size_t cchOut);
BOOL GetConfigFile(const TCHAR* wszFolder, __out_ecount(cchConfigFile) TCHAR* wszConfigFile, Size_t cchConfigFile);
BOOL GetCacheFile(const TCHAR* wszFolder, __out_ecount(cchCacheFile) TCHAR* wszCacheFile, Size_t cchCacheFile);
BOOL GetPluginCountersFile(const TCHAR* wszFolder, __out_ecount(cchCacheFile) TCHAR* wszCacheFile, Size_t cchCacheFile);
BOOL GetAllTimeCountersFile(const TCHAR* wszFolder, __out_ecount(cchFile) TCHAR* wszFile, Size_t cchFile);
BOOL GetSiteFolderPrefixByHostScheme(const TCHAR* wszHostname, const TCHAR* wszPort, __out_ecount(cchFolder) TCHAR* wszFolder, Size_t cchFolder);
BOOL GetFullPath(const TCHAR* wszFolder, const TCHAR* wszFilename, __out_ecount(cchFullPath) TCHAR* wszFullPath, Size_t cchFullPath);
BOOL NullTerminateString(__out_ecount(cchString) TCHAR* wszString, Size_t cchString, Size_t cchEnd);
BOOL CombineHostAndPort(__out_ecount(cchHostAndPort) TCHAR* wszHostAndPort, Size_t cchHostAndPort, const TCHAR* wszHost, const TCHAR* wszPort);

// IPC stands for Inter-Process Communication
const TCHAR* IPCNameFromAppPoolName(const TCHAR* tszAppPoolName);
BOOL SetBaseIPCName(const TCHAR* wszName);
BOOL GetIPCName(__out_ecount(cchFullName) TCHAR* wszFullName, Size_t cchFullName, const TCHAR* wszName, unsigned index);

#ifdef __TARGET_IIS__
// IIS-specific registry specific functions.
#define SSP_REGISTRY_PATH TEXT("SOFTWARE\\Microsoft\\SitemapPlugin")
#define SSP_REGISTRY_HIVE HKEY_LOCAL_MACHINE
#define SSP_REGISTRY_FOLDERPATH TEXT("DataDirectory")
#define SSP_REGISTRY_DATAFOLDERPATH TEXT("SitemapDataFolder")

BOOL PluginRegQueryValue(const TCHAR* wszSubkey, const TCHAR* wszValueName, __out_ecount(cchValue) TCHAR* wszValue, Size_t cchValue);
BOOL PluginRegQueryDWORD(const TCHAR* wszSubkey, const TCHAR* wszValueName, DWORD* pdw);

#endif

#if defined(__TARGET_APACHE__) || defined(__PUBLISH_MODE_MANUAL__)
// Apache specific configuration functions.
BOOL SetSiteMapPublishMode(DWORD dwMode);
DWORD GetSiteMapPublishMode();
BOOL SetSiteMapPingBackMode(DWORD dwMode);
DWORD GetSiteMapPingBackMode();
BOOL SetSiteMapHelpSSPMode(DWORD mode);
DWORD GetSiteMapHelpSSPMode();
#endif

#define SEC_PER_T32 300
#define T32_PER_DAY 288

#define DEADLINK_SIGNATURE MAX_UINT32

#define PORT_STR_SIZE 5		// it is # of digits in USHRT_MAX

#define HOST_MAPPING_FILE TEXT("HostMapping.txt")
#define BLACKLIST_FILE TEXT("disallow.txt")

class MyFileReader : public IFileReader
{
    UncompressingFileReader m_reader;
    unsigned m_version;
public:
    BOOL Initialize(const TCHAR* wszFile, const unsigned *pdwExpectedMagic);
    void Dispose() { m_reader.DoDispose(); }
    /*override*/ bool Read(__out_bcount(cb) void* pv, Size_t cb);
    /*override*/ unsigned Version() { return m_version; }
};


// test hooks
void SetTestTime(time_t ft);
time_t PluginTime();
void PluginGetLocalTime(struct tm*);
void PluginGetUTCTime(struct tm*);

