/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "basic_types.h"

typedef enum
{
    SitemapDontPublish = 0,
    SitemapPublishHidden = 1,
    SitemapPublishFull = 2
} SitemapPublishMode;


class SSPConfig
{
public:
#define TCHAR_PARAM(name,default) TCHAR wsz##name[MAX_PATH];
#define CHAR_PARAM(name,default) char sz##name[MAX_PATH];
#define BOOL_PARAM(name,default) BOOL f##name;
#define UINT_PARAM(name,default,lmin,lmax) unsigned u##name;
#define FLOAT_PARAM(name,default) float fl##name;
#include "SSPConfigList.h"
#undef TCHAR_PARAM
#undef CHAR_PARAM
#undef BOOL_PARAM
#undef UINT_PARAM
#undef FLOAT_PARAM

    SSPConfig();
    void Initialize();
};

BOOL ReadSSPConfig(const TCHAR* wszFile, SSPConfig* pConfig);
void PrintSSPConfig(FILE* fpOut, const SSPConfig* pConfig);
void WriteDefaultConfig(FILE* fpOut);
BOOL UpdateSSPConfig(const TCHAR* wszFile, const char* szName, const char* szValue);

extern BOOL g_fGZipSitemapsByDefault;
extern BOOL g_fPublishSitemapByDefault;
extern BOOL g_fPingSearchEngines;
extern BOOL g_fUseRegistryConfig;
extern BOOL g_fHelpSSPByDefault;

// setting name string literal constants
#define TCHAR_PARAM(name,default)          static const char* c_sz##name = #name;
#define CHAR_PARAM(name,default)           static const char* c_sz##name = #name;
#define BOOL_PARAM(name,default)           static const char* c_sz##name = #name;
#define FLOAT_PARAM(name,default)          static const char* c_sz##name = #name;
#define UINT_PARAM(name,default,lmin,lmax) static const char* c_sz##name = #name;
#include "SSPConfigList.h"
#undef TCHAR_PARAM
#undef CHAR_PARAM
#undef BOOL_PARAM
#undef UINT_PARAM
#undef FLOAT_PARAM


