/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "basic_types.h"

class Robots
{
    class RobotsData
    {
        static const char *szDisallow;
        static const char *szAllow;
        static const char *szUserAgent;
        static const char *szCrawlDelay;
        static const char *szSitemap;

        const static UInt32 c_maxRobotsUserAgents = 1;
        const static size_t c_maxRuleLength = 256;
        const static int c_maxRulesPerHost = 64 * 1024;

        static const char s_rgszUserAgents[c_maxRobotsUserAgents][256];
        static const size_t s_rgcbUserAgents[c_maxRobotsUserAgents];
        static const UInt32 s_numUserAgents = 1;
        static const UInt32 s_StarBotIndex = 0;
        
        enum RuleType
        {
            ALLOW,
            DISALLOW
        };
        
        ULONG m_countDisallow[c_maxRobotsUserAgents];
        ULONG m_countAllow[c_maxRobotsUserAgents];
        char *m_pchDisallow[c_maxRobotsUserAgents];
        char *m_pchAllow[c_maxRobotsUserAgents];
        bool m_fAgentFound[c_maxRobotsUserAgents];
        
        size_t m_cbDisallow;
        size_t m_cbAllow;
        char *m_disallowSpace;
        char *m_allowSpace;
        BOOL m_fOwnMemory;
        size_t m_disallowOffset;
        size_t m_allowOffset;
    
        BOOL Initialize(IAllocator* pAlloc, size_t);
        void ParseRobotsRules(const char *pchBuffer, size_t cbData);
        void AddRules(const UInt32 idUserAgent, const char *pchBuffer, const char *pchMax);
        void AddOneRule(const UInt32 idUserAgent, const char *pchBuffer, const char *pchMax, RuleType type);
        BOOL Finalize();
    public:
        RobotsData();
        ~RobotsData();
        BOOL Load(IAllocator* pAlloc, const TCHAR* wszFile, BOOL fRobots);
        bool UrlAllowed(const char * const szRelativeUrl);
    };

    BOOL m_fRobots;
    BOOL m_fNonRobots;
    RobotsData m_robots;
    RobotsData m_nonRobots;
public:
    BOOL Load(IAllocator* pAlloc, const TCHAR* wszRobotsFile, const TCHAR* wszNonRobotsFile);
    int Allowed(const char* szUrl);
    BOOL Allowed(const char* pcUrl, size_t cbUrl);
};

void WriteBlacklistFile(const TCHAR* wszFile);
BOOL UpdateBlacklistFile(const TCHAR* wszFile);

