/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#include "SnapshotFile.h"

size_t BaseSnapshotEntry::Write(CompressingFileWriter *pWriter)
{
    char temp[sizeof(BaseSnapshotEntry)];
    size_t cbOut = sizeof(temp);
    if (m_cbUrl > sizeof(temp) - sizeof(m_sig) - sizeof(m_sigBots) - sizeof(m_flags) - sizeof(m_lastMod) - sizeof(m_lastSeen) - sizeof(m_changePeriod) - sizeof(m_visitCount) - sizeof(m_cbUrl))
    {
        return 0;
    }

    char* p = temp;
    *((Signature*)p) = m_sig;  p += sizeof(m_sig);
    *((Signature*)p) = m_sigBots;  p += sizeof(m_sigBots);
    *((unsigned*)p) = m_flags;  p += sizeof(m_flags);
    *((unsigned*)p) = m_lastMod;  p += sizeof(m_lastMod);
    *((unsigned*)p) = m_lastSeen; p += sizeof(m_lastSeen);
    *((unsigned char*)p) = m_changePeriod;  p += sizeof(m_changePeriod);
    *((VisitCount*)p) = m_visitCount;  p += sizeof(VisitCount);
    *((UInt16*)p) = m_cbUrl;  p += sizeof(UInt16);
    if (m_cbUrl) // 0-length URL may be used as EOF marker
    {
        memcpy(p, m_pbUrl, m_cbUrl); p += m_cbUrl;
    }

    size_t cb = p - temp;
    if (NO_ERROR != pWriter->Write(temp, (DWORD)cb)) return 0;
    return cb;
}

bool BaseSnapshotEntry::Read(IFileReader* pFile)
{
    m_sigBots = 0;
    if (!pFile->Read(&m_sig, sizeof(Signature))) return false;
    if (pFile->Version() > 0)
    {
        if (!pFile->Read(&m_sigBots, sizeof(Signature))) return false;
        if (!pFile->Read(&m_flags, sizeof(m_flags))) return false;
    }
    if (!pFile->Read(&m_lastMod, sizeof(m_lastMod))) return false;
    if (pFile->Version() == 0)
    {
        m_flags = m_lastMod & SNAPSHOT_FLAG_FIRSTDISCOVERED; // v1`had this flag in m_lastMod
        m_lastMod &= ~SNAPSHOT_FLAG_FIRSTDISCOVERED;
    }
    if (!pFile->Read(&m_lastSeen, sizeof(m_lastSeen))) return false;
    if (!pFile->Read(&m_changePeriod, sizeof(m_changePeriod))) return false;
    if (!pFile->Read(&m_visitCount, sizeof(VisitCount))) return false;
    if (!pFile->Read(&m_cbUrl, sizeof(m_cbUrl))) return false;
    if (m_cbUrl > sizeof(m_pbUrl)) return false;
    if (m_cbUrl) // 0-length URL may be used as EOF marker
    {
        if (!pFile->Read(m_pbUrl, m_cbUrl)) return false;
    }
    return true;
}

int SnapshotFileCppLineCount()
{
    return __LINE__;
}
