/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "FilesCommon.h"
#include "CompressingWriter.h"
#include "SSPTypes.h"

// Magic number doubles as a file format version.
// These were generated randomly, although simply incrementing would be no worse.
static const unsigned SNAPSHOT_FILE_MAGIC_HEADER0 = 0xc6336a5c;
static const unsigned SNAPSHOT_FILE_MAGIC_HEADER1 = 0x4781c610;

static const unsigned c_rSnapshotMagic[] = { SNAPSHOT_FILE_MAGIC_HEADER0, SNAPSHOT_FILE_MAGIC_HEADER1, 0 };

#define SNAPSHOT_FLAG_FIRSTDISCOVERED 0x80000000
#define SNAPSHOT_FLAG_ROBOTS          0x40000000

class BaseSnapshotEntry
{
public:
    Signature      m_sig;
    Signature      m_sigBots;
    unsigned       m_flags;
    unsigned       m_lastMod;
    unsigned       m_lastSeen;
    unsigned char  m_changePeriod;
    VisitCount     m_visitCount;
    unsigned short m_cbUrl;
    char           m_pbUrl[2048];
    
    bool IsEmpty() { return !m_sig && !m_sigBots && !m_flags, !m_lastMod && !m_lastSeen && !m_changePeriod && !m_visitCount && !m_cbUrl; }
    BaseSnapshotEntry() : m_sig(0), m_sigBots(0), m_flags(0), m_lastMod(0), m_lastSeen(0), m_changePeriod(0), m_visitCount(0), m_cbUrl(0) { }
    
    bool Read(IFileReader* pReader);
    size_t Write(CompressingFileWriter *pWriter);
};
