/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "basic_types.h"
#include "shared.h"
#include "zlib.h"
#include "diskio.h"
#include "HeapInstance.h"

/*************************************************************************
**  Class: UncompressingFileReader
**
**  Description: Sequential file reader class.  Efficiently reads the file
**      sequentially using unbuffered and asynch I/O.  Can issue many reads 
**      in order to keep many disks busy and efficiently read the disk 
**      sequentially.
**
**************************************************************************/
class UncompressingFileReader
{
private:

    SEQREAD m_seqread;

    // buffer for compressed data
    UInt8 *m_compressedBuffer;
    UInt8 *m_uncompressedBuffer;

    z_stream m_strmUncompress;

    bool m_fCompression;

    UInt64 m_bytesRemaining;
    UInt64 m_fileSize;

    // Allocator for zlib
    HeapInstance* m_pAllocator;
    bool m_fDeleteAllocator;
    static void *zlib_alloc(void *opaque, UINT items, UINT size);
    static void zlib_free(void *opaque, void *address);

public:
    UncompressingFileReader();
    // pass in a HeapInstance to use
    UncompressingFileReader( HeapInstance* pAllocator );
    ~UncompressingFileReader();

    DWORD32 InitializeUncompression(void);

    DWORD32 DoInit(const TCHAR* szFile, 
        int maxReads = SEQREAD::c_maxReadsDefault, 
        DWORD32 readBufSize = SEQREAD::c_readBufSize, 
        BOOL readwrite = false);
    
    DWORD32 DoDispose();

    DWORD32 Reset(void);

    DWORD32 ResetUncompression(void);
    
    DWORD32 GetDataPointer( void ** ppvBuffer, DWORD dwNumBytes, DWORD* pcbRead=NULL );
    
    DWORD32 GetData( __out_bcount_opt(dwNumBytes) void * pvBuffer, DWORD dwNumBytes, DWORD* pcbRead=NULL );
    
    DWORD32 Skip(DWORD dwNumBytes);

    SEQREAD *GetReader(void) { return &m_seqread; }

    static DWORD32 UncompressFile(const TCHAR *pszCompressed, const TCHAR *pszUncompressed);

    const static UInt32 c_compressedBufferSize = 64 * 1024;
    const static UInt32 c_uncompressedBufferSize = 64 * 1024;
};


