/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#ifdef WIN32
#pragma warning (disable: 4619) // #pragma warning : there is no warning number 'NNNN'

#pragma warning (push)

// this define can be temporary removed to fix the build break
// over time we should fix the code and permanently disable crit section copying 

//////// #define DISABLE_COPY_OF_CRIT_SECTION 1 

//
// disable warnings that are generated by PSDK headers with /Wall
//

#pragma warning (disable: 4255)
#pragma warning (disable: 4668) // XXX is not defined as preprocessor macro, replacing with 0
#pragma warning (disable: 4820) // 'N' bytes padding added after data member 'XXX'
#pragma warning (disable: 4365) // conversion from 'LONG64' to 'DWORD64', signed/unsigned mismat
#pragma warning (disable: 4548) // malloc.h(245) & STL: expression before comma has no effect; expected expression with side-effect
#pragma warning (disable: 4995) // deprecated functions
#endif

#include "PAL.h"

#ifdef WIN32
#pragma warning (pop)
#endif

#if defined (ELEMENTCOUNT)
#undef ELEMENTCOUNT
#endif
#define ELEMENTCOUNT(x) (sizeof(x)/sizeof(x[0]))

#ifdef __TARGET_IIS__
typedef __int8 Int8;
typedef unsigned __int8 UInt8;
typedef __int16 Int16;
typedef unsigned __int16 UInt16;
typedef __int32 Int32;
typedef unsigned __int32 UInt32;
typedef __int64 Int64;
typedef unsigned __int64 UInt64;
typedef size_t Size_t;
#else
typedef char Int8;
typedef apr_byte_t UInt8;
typedef apr_int16_t Int16;
typedef apr_uint16_t UInt16;
typedef apr_int32_t Int32;
typedef apr_uint32_t UInt32;
typedef apr_int64_t Int64;
typedef apr_uint64_t UInt64;
typedef apr_size_t Size_t;
#ifndef MAX_PATH
#define MAX_PATH APR_PATH_MAX
#endif

// For now, we target little-endian machines
#if APR_IS_BIGENDIAN
#error This is not a little-endian platform.
#endif

#endif


#define MAX_UINT8  ((UInt8)-1)
#define MAX_UINT16 ((UInt16)-1)
#define MAX_UINT32 ((UInt32)-1)
#define MIN_INT32  ((Int32)0x80000000)
#define MAX_INT32  ((Int32)0x7FFFFFFF)      // 2147483647
#define MAX_UINT64 ((UInt64)-1)
#define MAX_INT64  0x7FFFFFFFFFFFFFFFi64
#define MAX_FLOAT  (3.402823466e+38F)
#define MAX_SIZE_T ((Size_t)((Size_t)0 - (Size_t)1))
#define PTR_FFFF ((void*)MAX_SIZE_T)

// A structure to represent a fixed-size chunk of
// memory
struct SIZED_STRING
{
    union
    {
        const UInt8 *pbData;
        const char  *pcData;
    };
    Size_t cbData;
};

// A helper macro for defining a SIZED_STRING as part of a constant
#define INLINE_SIZED_STRING(str) { (const UInt8 *)str, sizeof(str) - 1 }

//  Structure to store a wchar_t version of a user dictionary word in memory
struct WCHAR_SIZED_STRING
{
    wchar_t* pData;
    Size_t cchData;
};


// A helper macro for defining a SIZED_STRING as part of a constant
#define INLINE_WCHAR_SIZED_STRING(wstr) { wstr, sizeof(wstr)/sizeof(wstr[0]) - 1 }

//  Structure to store a wchar_t version of a user dictionary word in memory
struct WCHAR_SIZED_STRING_CONST
{
    const wchar_t* pData;
    Size_t cchData;
};

typedef WCHAR_SIZED_STRING_CONST SIZED_STRING_W;


// A utility class for creating temporary SIZED_STRINGs
class CStackSizedString : public SIZED_STRING
{
public:
    // Null-terminated input
    CStackSizedString(const char *szValue)
    {
        pbData = (const UInt8 *)szValue;
        cbData = strlen(szValue);
    }

    // Name/size pair
    CStackSizedString(
        const UInt8 *pbValue,
        Size_t cbValue)
    {
        pbData = pbValue;
        cbData = cbValue;
    }

    // Name/size pair
    CStackSizedString(
        const char *pcValue,
        Size_t cbValue)
    {
        pcData = pcValue;
        cbData = cbValue;
    }

private:
    // prevent heap allocation
    void *operator new(Size_t);
};

// A utility class for creating temporary WCHAR_SIZED_STRINGs
class CStackSizedWString : public WCHAR_SIZED_STRING_CONST
{
public:
    // Null-terminated input
    CStackSizedWString(const wchar_t *wzValue)
    {
        pData = wzValue;
        cchData = wcslen(wzValue);
    }

    // Name/size pair
    CStackSizedWString(
        const wchar_t *wzValue,
        Size_t cchValue)
    {
        pData = wzValue;
        cchData = cchValue;
    }

private:
    // prevent heap allocation
    void *operator new(Size_t);
};

typedef CStackSizedWString CStackSizedStringW;


// DDWORD is used to easily access a 64bit number as both a 64bit
//   and as two 32bit numbers
typedef union
{
    // Bugbug: Need to handle different endians if we target non-intel platforms.
    struct
    {
        DWORD	low;
        DWORD	high;
    } dw;
    DWORD64	ddw;
} DDWORD;

///////////////////////////////////
#ifdef WIN32
#pragma warning(push)
#pragma warning (disable: 4201) // nonstandard extension used : nameless struct/union
#endif

typedef union tagFAInt64
{
    UInt64  n64;
    Int64   i64;
    UInt8   nBytes[8];
    FILETIME  ft;

    struct
    {
        UInt32 nData;
        UInt32  nCount;
    };
    struct
    {
        Int32  i32_low;
        Int32  i32_hi;
    };
    struct
    {
        UInt32   n32_low;
        UInt32   n32_hi;
    };

    tagFAInt64():n64(0){};
    tagFAInt64( UInt64 nVal ):n64(nVal){};
    tagFAInt64( Int64  iVal ):i64(iVal){};
    tagFAInt64( UInt32 nHigh, UInt32 nLow ): n32_low(nLow), n32_hi(nHigh){};
    tagFAInt64( Int32  iHigh, Int32  iLow ): i32_low(iLow), i32_hi(iHigh){};
    //tagFAInt64( DWORD  nHigh, DWORD  nLow ): n32_low(nLow), n32_hi(nHigh){};
    tagFAInt64( const FILETIME&  rftSrc ): ft(rftSrc){};

} FAInt64;  // flexible access 64 bit integer

typedef union tagFAInt32
{
    UInt32 n32;
    Int32   i32;
    UInt8   nBytes[4];
    struct
    {
        Int16  i16_low;
        Int16  i16_hi;
    };
    struct
    {
        UInt16   n16_low;
        UInt16   n16_hi;
    };
    tagFAInt32():n32(0){};
    tagFAInt32( UInt32 nVal ): n32(nVal){};
    tagFAInt32( Int32  iVal ): i32(iVal){};

} FAInt32;  // flexible access 32 bit integer

class IAllocator
{
public:
    virtual void *Allocate(size_t cb) = 0;
};

class MallocAllocator : public IAllocator
{
public:
    void* Allocate(size_t cb) { return malloc(cb); }
};

class ArenaAllocator : public IAllocator
{
    BYTE* m_pb;
    size_t m_cb;
public:
    void Initialize(void* p, size_t cb) { m_pb = (BYTE*)p;  m_cb = cb; }
    void* Allocate(size_t cb)
    {
        if (cb > m_cb) return NULL;
        void* pRet = m_pb;
        m_pb += cb;
        m_cb -= cb;
        return pRet;
    }
    size_t Left() { return m_cb; }
};


#ifdef WIN32
#pragma warning(pop)

//
// disabling a couple of warnings that show up with /Wall and are pretty much useless -- they come mostly from macros
//
#pragma warning (disable: 4514) // unreferenced inline function has been removed
#pragma warning (disable: 4820) // 'N' bytes padding added after data member 'XXX'
#pragma warning (disable: 4265) // class has virtual functions, but destructor is not virtual
#pragma warning (disable: 4668) // XXX is not defined as preprocessor macro, replacing with 0
#pragma warning (disable: 4711) // function XXX selected for automatic inline expansion
#pragma warning (disable: 4548) // malloc.h(245) & STL: expression before comma has no effect; expected expression with side-effect
#pragma warning (disable: 4127) // conditional expression is constant
#pragma warning (disable: 4189) // local variable is initialized but not referenced
#endif
