/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "basic_types.h"

class BloomFilter
{
public:
    BloomFilter();
    ~BloomFilter();

    bool Contains(size_t length, const void* p) const;
    void Add(size_t length, const void* p);

    void Reset();
    bool Init(void* p, size_t cb, unsigned cHashIn = 7);
    
private:    
    UInt64 CalculateHash(
        const UInt64 base,
        const size_t length, 
        const void* buffer
        ) const;

    bool GetBit(
        const UInt64 bitIndex
        ) const;

    void SetBit(
        const UInt64 bitIndex
        );


    size_t cBit;
    size_t cUInt64;//Keep around instead of calculating it over.

    UInt32 cHash;
    UInt64* bit;
};

