/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "basic_types.h"
#include "PALSync.h"

class Counter
{
public:
    volatile UInt32 m_val;
    Counter() { m_val = 0; }

    void Increment() { PALInterlocked::Increment(&m_val); }
    void Reset() { PALInterlocked::Set(&m_val, 0); }
};

typedef struct {
#define COUNTER(name) Counter c##name;
#include "counterlist.h"
#undef COUNTER
} Counters;

void ResetCounters(Counters* pCounters);
BOOL ReadCounters(const TCHAR* wszFile, Counters* pCounters);
void AddCounters(Counters* pCounters, const Counters *pAdd);
BOOL AddCountersToFile(const TCHAR* wszFile, const Counters* pCounters);

class NotificationCounters
{
    BOOL m_fInitialized;
    time_t m_t;
    TCHAR m_wszFile[MAX_PATH];
    void Initialize();
    void Save();
public:
    unsigned m_c[100];
    NotificationCounters();
    void Inc(unsigned);
    void Tick();
};

