/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "PALSync.h"
#include "SSPConfig.h"

#define SITEMAP_PREFIX "trafficbasedssp"
#define SITEMAP_DEADLINK "deadlink"
#define SITEMAP_DELTA "delta"
#define SITEMAP_SUFFIX "sitemap"

#define SITEMAP_NAME SITEMAP_PREFIX SITEMAP_SUFFIX
#define DEADLINK_SITEMAP_NAME SITEMAP_PREFIX SITEMAP_DEADLINK SITEMAP_SUFFIX
#define DELTA_SITEMAP_NAME SITEMAP_PREFIX SITEMAP_DELTA SITEMAP_SUFFIX
#define DELTA_DEADLINK_SITEMAP_NAME SITEMAP_PREFIX SITEMAP_DEADLINK SITEMAP_DELTA SITEMAP_SUFFIX

// For easier testing, wszRobotsFolder is optional - if NULL, the wwwroot value from DataFolder\config.ini is used.
// The real service passes in a verified wszRobotsFolder value because the one from config.ini is untrusted.
// szSubPath (optional, usually empty) is the www path of the site's sub-root (where the sitemap goes) relative to the host's root (where robots.txt goes)
// If specified, szSubPath must begin with a slash and end without a slash, and all slashes in szSubPath are expected to be forward (even on Windows)
void DoOneSite(const TCHAR* wszDataFolder, const TCHAR* wszRobotsFolder, const char* szSubPath, PALEvent *pStopEvent);
BOOL OpenMergeServiceLog(const TCHAR* wszPath, FILE**);
void CloseMergeServiceLog(FILE*);
Size_t FolderPrefixSize(const TCHAR* wszFolder);

#define MERGE_OPTION "-merge"

// Used on master machine in multi-server scenario after DeltaSnapshot files have been collected in wszInputFolder
int CommandLineMerge(int argc, TCHAR **wzArgv);

extern BOOL g_fTestNoFullSitemap; // used by automated test

