/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#include "RobotsUpdater.h"
#include "SSPCommon.h"
#include "ServiceLogging.h"
#include "PALFile.h"
#include "PALString.h"

static BOOL ScanRobotsFile(const TCHAR* wszFile, const char* szLine, BOOL* pfContains, BOOL* pfMissingEOL)
{
    size_t cchLine = strlen(szLine);
    FILE* fp = NULL;

    if (0 != PALFile::Open(&fp, wszFile, TEXT("rt")) || !fp)
    {
        return FALSE;
    }

    *pfContains = FALSE;
    *pfMissingEOL = FALSE;
    while (!feof(fp))
    {
        char s[1024];
        if (!fgets(s, sizeof(s), fp)) continue;
        if (!s[0]) break;
        size_t cch = strlen(s);
        *pfMissingEOL = ('\n' != s[cch-1]);
        if (cch >= cchLine && !memcmp(szLine, s, cchLine)) // lazy - in theory szLine could be suffix of s, but more likely s has trailing \r\n
        {
            *pfContains = TRUE;
        }
    }
    fclose(fp);
    return TRUE;
}

BOOL AddSitemapToRobots(const TCHAR* wszWWWRoot, const TCHAR* wszTempFolder, const char* szSitemap, const TCHAR* szRobotsTxtFileName)
{
    UNREFERENCED_PARAMETER(wszTempFolder);
    FILE* fp = NULL;
    BOOL fAppend = FALSE, fMissingEOL = FALSE;
    char szLine[1024];
    TCHAR wszRobots[MAX_PATH];

    if (0 >= SPRINTF_S(szLine, ARRAYSIZE(szLine), "Sitemap: %s", szSitemap)) SGS_ERROR;
    
    if (!GetFullPath(wszWWWRoot, szRobotsTxtFileName, wszRobots, ARRAYSIZE(wszRobots))) SGS_ERROR;

    // check if it already contains
    if (PALDirectory::PathExists(wszRobots))
    {
        BOOL fContains;
        if (!ScanRobotsFile(wszRobots, szLine, &fContains, &fMissingEOL)) // lazy - only checking for the exact casing/whitespace we use.  Worst case we add a redundant line.
        {
            SGS_ERROR; // cannot even read robots.txt - don't mess with it any further
        }
        if (fContains)
        {
            return TRUE;
        }
        fAppend = TRUE;
    }

    if ((0 != PALFile::Open(&fp, wszRobots, fAppend ? TEXT("at") : TEXT("wt"))) || !fp) SGS_ERROR;
    if (0 >= fprintf(fp, "%s%s\r\n", fMissingEOL ? "\r\n" : "", szLine)) SGS_ERROR;
    fclose(fp);
    return TRUE;

Exit:
    return FALSE;
}
