/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include <PAL.h>
#include "counters.h"
#include "SSPConfig.h"
#include "SSPTypes.h"
#include "CompressingWriter.h"
#include "FileAccess.h"

class SitemapOutput
{
    FILE* m_fpIndex;
    
    // child sitemap
    CompressingFileWriter* m_pContentWriter;
    CompressingFileWriter* m_pDeadlinkWriter;
    size_t m_cbIndex;
    size_t m_cbContentChild;
    size_t m_cEntriesInCurContentChild;
    
    size_t m_cbDeadlinkChild;
    size_t m_cEntriesInCurDeadlinkChild;

    BOOL m_fError;

    unsigned m_iContentFile;
    unsigned m_iFirstContentFile;
    
    unsigned m_iDeadlinkFile;
    unsigned m_iFirstDeadlinkFile;

    TCHAR m_wszCurContentFile[MAX_PATH];
    TCHAR m_wszCurDeadlinkFile[MAX_PATH];

    TCHAR m_wszTempIndexFile[MAX_PATH];
    TCHAR m_wszIndexFile[MAX_PATH];
    
    TCHAR m_wszTempFolder[MAX_PATH];
    TCHAR m_wszOutFolder[MAX_PATH];

    char m_szContentName[80];
    char m_szDeadlinkName[80];

    char m_szTimeNow[80];
    char m_szWWWHost[200];
    char m_szSubPath[MAX_PATH];

    // this is hash of computer name/host name of machine running plugin
    UInt64 m_uHostHash;
    size_t m_cbWWWHost;

    BOOL m_fIncremental;
    BOOL m_gzip;
    BOOL m_fCrossServer;

    void Free();

    BOOL GetChildFileName(__out_ecount(cchName) char* szName, size_t cchName, const char* prefix, unsigned i);
    
    BOOL NewChildSitemap(CompressingFileWriter** pFileWriter, const char* szChild, __out_ecount(cCurFile) TCHAR* wszCurFile, size_t cCurFile);
    BOOL NewChildSitemap(BOOL fDeadlink);

    BOOL CloseChild(BOOL fDeadlink);
    
    int PrintToSitemap(CompressingFileWriter* pFileWriter, const char* format, ...);

    BOOL WriteHeader(CompressingFileWriter* pFileWriter);
    BOOL WriteEntry(Signature sig, Signature sigBots, const char* pcUrl, size_t cbUrl, unsigned ft32LastMod, BOOL fLastModRobots, const char* szChangeFreq, float flPriority, float flSSPPriority);
    BOOL WriteFooter(BOOL fDeadlink);   

    // Set Everyone read access for allowing anonymous access via http
    BOOL SetFileAccessForWebAccess(const TCHAR* wszFilePath);

    // Move sitemap file with permission fix to be accessible via HTTP request
    BOOL MoveSiteMapFile(const TCHAR* wszSrcFile, const TCHAR* wszDstFile);
    
    // Move chlid sitemap files to the wwwroot
    BOOL MoveChildSiteMapFiles(const char* szSiteMapName, unsigned iFirstFileIdx, unsigned iLastFileIdx);

    // Util function to get the pointer to one of two child file writer to help
    // with managing the single code for both content sitemaps and deadlink sitemaps.
    CompressingFileWriter** GetChildFileWriter(BOOL fDeadlink);

public:
    SitemapOutput();
    ~SitemapOutput();
    BOOL Initialize(const char* szName, const char* szDeadlinkName, const TCHAR* wszTempFolder, const TCHAR* wszOutFolder, const char* szWWWHost, const char* szSubPath, BOOL gzip, BOOL fIncremental, BOOL fCrossServer);
    BOOL AddEntry(Signature sig, Signature sigBots, const char* pcUrl, size_t cbUrl, unsigned t32LastMod = 0, BOOL fLastModRobots = FALSE, const char* szChangeFreq = NULL, float flPriority = 0.5, float flSSPPriority = 0);
    BOOL Close(const Counters* pCounters = NULL, const SSPConfig* pConfig = NULL, BOOL fKeepIndexFile = FALSE);

    static BOOL GetSitemapFilePath(__out_ecount(rchPath) TCHAR* wszPath, size_t rchPath, const TCHAR* wszFolder, const char* szName);

    static BOOL EncodeURL(const char* pcUrl, size_t cbUrl, __out_bcount_part(*pcbOut,*pcbOut) char* pcOut, size_t* pcbOut);
};

