/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/

#pragma once
#include "PluginLibCommon.h"
#include "PALThread.h"
#include "PlatformSpecificPluginGlobals.h"
#include "PluginShared.h"

class PluginGlobals // platform-specific per-process
{
    friend class PluginSite;
    PALThread* m_pThreadPool;
    VirtualAllocator *m_pAllocator;
    CritSecManager *m_pLockManager;

    CRITSEC* m_pcs; // dedicated critical section for PluginShared::GetSite() because everything goes through it
    PluginShared* m_pShared;
    BOOL m_fWorkAcrossProcesses;
    BOOL m_fIsInitialized;

public:
    PluginGlobals() { memset(this, 0, sizeof(*this)); }
    BOOL Initialize(const ConfigPluginGlobals &config, void *pPlatformMemPool);
    BOOL InitializeChild(void *pPlatformMemPool);
    BOOL IsInitialized() { return m_fIsInitialized; };
    void Free(void *pPlatformMemPool);
    void FreeChild(void *pPlatformMemPool);
    PluginSite* GetSite(WSFilter *pFilter);
    ICriticalSection* CritSecPtr(int i) { return m_pLockManager->CritSecPtr(i); }
};
