/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/

#include "PluginLibCommon.h"
#include "PALFile.h"
#include "PALString.h"


#if DBG
void DebugString(const char* message)
{
    char szTemp[1024];
    Size_t cb = strlen(message);
    if (cb > sizeof(szTemp) - 2)
        cb = sizeof(szTemp) - 2;
    memcpy(szTemp, message, cb);
    szTemp[cb] = '\n';
    szTemp[cb+1] = '\0';
    OutputDebugStringA(szTemp);
}
#endif

void LogMessage(const TCHAR * wszLogFile, const char* msg)
{
    if ('\0' == wszLogFile[0])
    {
        return;
    }
    FILE* fp = NULL;
    if (0 != PALFile::Open(&fp, wszLogFile, TEXT("at")))
    {
        DebugString("fopen() failed");
        return;
    }

    struct tm tms;
    PluginGetLocalTime(&tms);
    fprintf(fp, "%04u/%02u/%02u %02u:%02u:%02u " STRFMT_TS " pid %u %s\n", 1900 + tms.tm_year, 1 + tms.tm_mon, tms.tm_mday, tms.tm_hour, tms.tm_min, tms.tm_sec, g_tszUserName, GetCurrentProcessId(), msg);
    fclose(fp);
}

const char* UserAgentType::ToString(UserAgentType::Type userAgentType)
{
    int id = (int)userAgentType;
    if (id < g_NumberOfUserAgentTypeStrings)
    {
        return g_UserAgentTypeStrings[id];
    }
    return g_UserAgentTypeStrings[g_NumberOfUserAgentTypeStrings - 1];
}

const UserAgentType::Type UserAgentType::GetUserAgentType(const char* szUserAgent)
{
    for (Size_t i = 0; i < g_NumberOfUserAgentTypeStrings; i++)
    {
        if (strstr(szUserAgent, g_UserAgentTypeStrings[i]))
        {
            return (UserAgentType::Type) i;         
        }
    }
    return UserAgentType::Unknown;
}