/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once
#include "basic_types.h"
#include "counters.h"
#include "SSPCommon.h"
#include "SSPTypes.h"

#define HOST_PATH_LENGTH 7 + MAX_PATH + PORT_STR_SIZE + 2
#define MAX_USERNAME 255
#define RESPONSE_FLAG_BOT 1
#define RESPONSE_FLAG_COMPRESSED 2
#define CACHE_REC_BYTES (sizeof(Hash) + sizeof(Signature) + sizeof(FileWriteTime) + sizeof(VisitCount))

static TCHAR g_tszUserName[MAX_USERNAME + 1] = TEXT("");
static volatile UInt32 s_cPendingWork = 0;

typedef struct
{
    Signature sig;
    FileWriteTime lastWrite;
    VisitCount visitCount;
} CacheEntry;


typedef struct
{
    char buf[CACHE_REC_BYTES * 1024];
    Size_t cb;
    FILE* fp;
} CacheSaveContext;


class IPluginLog
{
public:
    virtual void PluginLog(const char* szFormat, ...) = 0;
};

static const char* g_UserAgentTypeStrings[8] = 
{ 
    "Bingbot",
    "Googlebot",
    "OtherBot",
    "MSIE",
    "Chrome",
    "FireFox",
    "OtherBrowser",
    "Unknown"
};

static const int g_NumberOfUserAgentTypeStrings = (sizeof g_UserAgentTypeStrings) / (sizeof g_UserAgentTypeStrings[0]);

namespace UserAgentType
{
    // This list should be synced with g_UserAgentTypeStrings (PluginLibCommon.cpp)
    // Please make sure when you change enum, change g_UserAgentTypeStrings and ToString() implementation as well
    enum Type
    {
        BingBot,            // = "Bingbot"
        GoogleBot,          // = "Googlebot"
        OtherBot,           //

        IE,                 // = "MSIE"
        Chrome,             // = "Chrome"
        FireFox,            // = "Firefox"
        OtherBrowser,       //    

        // Total count of types
        Unknown
    };

    const char* ToString(UserAgentType::Type userAgentType);

    const UserAgentType::Type GetUserAgentType(const char* szUserAgent);

}

#define CALLEE_ERROR { pLog->PluginLog("error at line %u", __LINE__); goto Exit; }

void LogMessage(const TCHAR * wszLogFile, const char* msg);

#if DBG
void DebugString(const char* message);
#else
#define DebugString(a)
#endif

