/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/

#include "PALFile.h"
#include "PALString.h"
#include "UserAgents.h"


BOOL UserAgents::Allowed(const char* szUserAgent)
{
    BOOL fMatch = FALSE;
    for (Size_t i = 0; i < m_cAgents; i++)
    {
        if (strstr(szUserAgent, m_rszAgents[i]))
        {
            if (m_rfAllowed[i])
            {
                fMatch = TRUE;
            }
            else
            {
                return FALSE;
            }
        }
    }
    return fMatch;
}

BOOL UserAgents::Load(IAllocator* pAlloc, const TCHAR* wszFile, IPluginLog* pLog)
{
    m_cAgents = 0;
    FILE* fp = NULL;
    BOOL fRet = FALSE;
    if (0 != PALFile::Open(&fp, wszFile, TEXT("rt")))
    {
        return FALSE;
    }

    while (!feof(fp))
    {
        char s[1024];
        s[0] = '\0';
        if (!fgets(s, sizeof(s), fp) || !s[0])
        {
            break;
        }
        if ('#' == s[0])
        {
            continue;
        }

        if (m_cAgents >= MAX_USER_AGENTS) CALLEE_ERROR;

        const char* pcTab = strchr(s, '\t');   if (!pcTab) CALLEE_ERROR;

        if (1 != SSCANF_S(pcTab + 1, "%u", m_rfAllowed + m_cAgents))  CALLEE_ERROR;

        char* szName = (char*)pAlloc->Allocate(pcTab - s + 1);  if (!szName)  CALLEE_ERROR;
        if (pcTab > s) memcpy(szName, s, pcTab - s);
        szName[pcTab - s] = '\0';
        m_rszAgents[m_cAgents] = szName;
        m_cAgents++;
    }

    fRet = TRUE;
Exit:
    if (fp) fclose(fp);
    // note: not bothering to free allocated memory => can leak
    return fRet;
}