/***********************************************************************
*                                                                      *
*   Copyright (c) Microsoft Corporation                                *
*                                                                      *
*   All rights reserved.                                               *
*                                                                      *
*   Licensed under the Apache License, Version 2.0 (the "License");    *
*   you may not use this file except in compliance with the License.   *
*   You may obtain a copy of the License at                            *
*   http://www.apache.org/licenses/LICENSE-2.0                         *
*                                                                      *
*   THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR         *
*   CONDITIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING       *
*   WITHOUT LIMITATION ANY IMPLIED WARRANTIES OR CONDITIONS OF TITLE,  *
*   FITNESS FOR A PARTICULAR PURPOSE, MERCHANTABLITY OR                *
*   NON-INFRINGEMENT.                                                  *
*                                                                      *
*   See the Apache Version 2.0 License for specific language           *
*   governing permissions and limitations under the License.           *
*                                                                      *
***********************************************************************/
#pragma once

#include "PAL.h"
#include "basic_types.h"

#ifdef __TARGET_IIS__
#include <Httpfilt.h>
#else
#include <httpd.h>
#include <util_filter.h>
#endif

// The web server filter class
class WSFilter
{
public:
#ifdef __TARGET_IIS__
    WSFilter(PHTTP_FILTER_CONTEXT p);
#else
    WSFilter();
    void SetFilter(ap_filter_t *pFilter, apr_bucket_brigade *pBrigade);
#endif

    ~WSFilter();

    void *GetContext() const;

    void SetContext(void *pContext);

    void *GetPlatformMemPool();

    BOOL GetNextDataSet(void *&pData, Size_t &cbSize);

    BOOL NoMore() const;

    // nLen is the length of tszString. If it returns TRUE,
    // it will return with the length (not including null-term)
    // of the null-terminated string.

    BOOL GetApplPhysicalPath(TCHAR *tszString, Size_t &nLen);

    BOOL GetContentType(__out_ecount(nLen) TCHAR *tszString, Size_t &nLen);

    BOOL GetServerName(TCHAR *tszString, Size_t &nLen);

    BOOL GetServerPort(TCHAR *tszString, Size_t &nLen);
        
    BOOL GetHTTPStatus(int &nStatus);

    BOOL GetURL(TCHAR *tszString, Size_t &nLen);

    BOOL GetUserAgent(char *szString, Size_t &nLen);

    BOOL IsHTTPCompressedContent() const;

private:
#ifdef __TARGET_IIS__
    BOOL GetServerVariable(LPSTR lpszVariableName, TCHAR *tszString, Size_t &nLen);
    BOOL GetServerVariableA(LPSTR lpszVariableName, char *szString, Size_t &nLen);

    BOOL GetHeaderVariable(LPSTR lpszVariableName, __out_ecount(nLen) TCHAR *tszString, Size_t &nLen);

    BOOL StringContains(const void* pv, size_t cb, const char* szSearch) const;
    BOOL IsCompressed() const;

    BOOL m_fCompressed;
public:
    PHTTP_FILTER_CONTEXT     m_pfc;
    PHTTP_FILTER_SEND_RESPONSE m_pResponse;
    PHTTP_FILTER_RAW_DATA m_pDataBuffer;
    BOOL m_fNoMore;
#else
    BOOL CopyCheckString(TCHAR *tszString, Size_t &nLen, const char *szSrc);

    ap_filter_t             *m_pfc;
    apr_bucket_brigade      *m_pBrigade;
    apr_bucket              *m_pCurrBucket;
    void                    *m_pContext;
    BOOL                     m_fNoMore;
#endif
};
